<?php if (!defined('PLX_ROOT')) exit;

##############################################################################################
#
# plnKikan. Plugin de gestion de choix de dates convenant à un maximum de participants.
# Auteur reprise  v1.2 (>=) : 24/03/2019..., Thomas Ingles
# Auteur originel v1.1 : 25/02/2015, Gari
#
# Principe
# --------
# La page d'administration permet de créer un nouveau Kikan. Cela se traduit par :
# - une description
# - un titre
# - une liste de dates et d'horaires
#
# Côté site, une (fausse) page statique spécifique permet d'afficher la liste de tous les Kikan.
# L'utilisateur n'a plus qu'à en choisir un, qui affiche le formulaire de choix.
#
# Les Ki
# --------
# Un Ki contient, en sus des informations de paramétrage définis plus haut, l'ensemble
# des informations fournies par les participants.
# Un tableau avec un enregistrement par participant, chaque enregistrement ayant une valeur
# numérique par date/horaire ou objet/element paramétré par le ki.
# 0 : kikan : non présent   : kikoi : J'ai pas
# 1 : kikan : peut-être     : kikoi : J'ai peut-être
# 2 : kikan : peut venir    : kikoi : j'ai
#
# Optionnel
# ---------
# Ce plugin peut être couplé à adhesion ou plnExtendedJurisdiction + lockArticles pour
# rendre privés certains Kis, voire les associer à des profils plus fins. C'est cependant un option :
# si plnExtendedJurisdiction ou adhesion sont absents, il y aura aucune privatisation possible.
#
##############################################################################################
class plnKikan extends plxPlugin
{
	static $TimeStamp;
	static $Date;
	static $Time;
	public $DataDir;
	public $DataFile;
	public $PluginDir;
	public $aKikan;
	public $kikanId = '';
	static $adhesion;#v1.2
	static $lockArticles;#v1.2
	static $isPrivacyPossible;
	public $isAccesInterdit;# true si le visiteur n'a pas les droits d'accès
	static $isStaticPages;# true si le module plnStaticPages est correctement chargé
	private static $nbInstances=0;
#MyMultiLinguev2.2
	static $isMultiLingue;# true si le module plxMyMultilingue est chargé avant
	public $lang = '';# sous dossier mml fr/
	public $lnGet = '';# langue en cours (capturé)
	public $aLangs = false;
	public function onUpdate(){# BEP #non testé
		$this->OnActivate();# Créer le dossier des données
		# Les anciens répertoires (2.1.x)
		$dataDirO   = PLX_ROOT.PLX_CONFIG_PATH.DIRECTORY_SEPARATOR.'plugins'.DIRECTORY_SEPARATOR;
		$dataFileO  = $dataDirO.__CLASS__.'_data.xml';
		if(file_exists($dataFileO)){
			$num = 0;
			while(1)
			{
				$newfile = str_replace('a.x', 'a_'.$num.'_old_'.time().'.x', $dataFileO);# never a.x (on loadkikan all not rename filename)
				# Verifie si cette sauvegarde n'existe pas déja (au cas ou, mais cela m'étonnerais)
				if(!is_file($newfile ))
				{
					# Copie vers plnKikan/data_old_########.xml (backup)
					if(copy($dataFileO, $newfile))# fichier a supprimer (todo msg a la install de PluXml)
					{
						# Garde le nouveaux chemin
						$dataFile = $this->DataFile;
						# Shift to load all olds datas
						$this->DataFile = $newfile;
						# Lit & charge les anciennes données + les choix
						$this->loadKikan(true);# + DATAS
						# Restaure pour générer les nouveaux fichiers
						$this->DataFile = $dataFile;
						# Modifie les états #brouillon == '0' ou #publié == '1'
						foreach($this->aKikan as $Id => $kikan)
							$this->aKikan[$Id]['status'] = $this->aKikan[$Id]['status'] != 'publié'?'0':'1';
						# Les Choix sont séparés : configuration/plugins/plnKikan/datas_#.xml (v2.2.0)
						$this->saveKikan('choix');# Enregistre les 'choix' des kikans
						# Enregistre les 'model' (configuration/plugins/plnKikan/data.xml)
						$this->saveKikan();
						# Supprime l'ancien fichier (configuration/plugins/plnKikan_data.xml)
						unlink($dataFileO);
						plxMsg::Info(__CLASS__ . ' ' . $this->getLang('L_MAJ_SUCCESS'));#M.A.J. réussie : anciennes données récupérées');
						break;
					}
				}
				$num++;
				if($num>10)
				{# Erreur de M.A.J. : impossible de récupérer les anciennes données');
					plxMsg::Error(__CLASS__ . ' ' . $this->getLang('L_MAJ_ERROR'));
					break;
				}
			}#Fin while 1
		}
		return array('cssCache' => true);
	}#$updAction = $instance->onUpdate(); to update cssCache
	public function checkHtaccess($fldr = ''){#v 2.2.0 Méthode appelée quand on a un fichier update : pour récuperer les acien kikians
		$file = PLX_ROOT.PLX_CONFIG_PATH.'.htaccess';
		$newfile = $this->DataDir.DIRECTORY_SEPARATOR.$fldr.'.htaccess';
		if(!is_file($newfile ))# .htaccess check
			if(!copy($file, $newfile)) @$_SESSION['error'] .= "La copie du fichier <i>$file</i> vers <i>$newfile</i> a échoué...\n";
	}
	public function OnActivate(){# Méthode appelée quand on active le plugin : pour créer les répertoires
		if(!is_dir($this->DataDir))#data dir check
			mkdir($this->DataDir);
		$this->checkHtaccess();
	}

	##############################################################################################
	#
	# Le constructeur
	#
	##############################################################################################
	public function __construct($default_lang)
	{
		# On vérifie que le plugin n'est pas chargé plusieurs fois à tort
		self::$nbInstances++;
		if(self::$nbInstances !== 1){
			echo '<!-- '.__CLASS__.' '.$this->getLang('L_ERROR_MULTI_INSTANCES').' '.self::$nbInstances.'x </br> -->';
			#var_dump(__CLASS__.'->'.__FUNCTION__.'()');$e = new \Exception;var_dump($e->getTraceAsString());
		}

		if(defined('PLX_MYMULTILINGUE')) {# Si plugin plxMyMultilingue présent
			$lang = plxMyMultiLingue::_Lang();# Récupération de la langue en cours
			$get = array_keys($_GET);
			$get = empty($get)?'':$get[0];# Garde le 1er param
			if(!empty($lang)) {
				$this->lang = $lang.DIRECTORY_SEPARATOR;
				#$this->default_lang = $default_lang = $lang;# Langue Dynamique
				if(isset($_SESSION['default_lang']) AND $_SESSION['default_lang']!=$lang)
					$this->lnGet = $lang . DIRECTORY_SEPARATOR;
				elseif(preg_match('~^'.$lang.'/~',$get))#find ##/ 1er get
					$this->lnGet = $lang . DIRECTORY_SEPARATOR;
				//~ elseif(preg_match('~^'.$lang.'/~',plxUtils::getGets()))#find ##/ getGets
					//~ $this->lnGet = $lang . DIRECTORY_SEPARATOR;
			}
			$lang = plxMyMultiLingue::_Langs();# récupération du tableau des langues activées
			$this->aLangs = empty($lang) ? $this->aLangs : explode(',', $lang);
		}

		parent::__construct($default_lang);

		# Date du jour (à définir impérativement une seule fois pour éviter un bug au passage de minuit)
		self::$TimeStamp			  = date('Y-m-d H:i:s');
		list(self::$Date,self::$Time) = explode(' ',self::$TimeStamp);

		# Les répertoires
		$this->PluginDir = PLX_PLUGINS.__CLASS__.DIRECTORY_SEPARATOR;
		$this->DataDir   = PLX_ROOT.PLX_CONFIG_PATH.'plugins'.DIRECTORY_SEPARATOR.__CLASS__.DIRECTORY_SEPARATOR;
		$this->DataFile  = $this->DataDir.$this->lang.'data.xml';# (/###)

		# Les droits d'accès à la page de configuration du plugin
		$this->setConfigProfil(PROFIL_ADMIN);

		if(class_exists('plnToolBox_v2')) {#plnToolBoxes (plugin)
			# On vérifie que le plugin technique facultatif lockArticles est installé et chargé avant plnKikan
			self::$lockArticles = plnToolBox_v2::checkPlugin('lockArticles',__CLASS__,'before',false);#v1.2

			# On vérifie que le plugin technique facultatif adheion est installé et chargé avant plnKikan
			self::$adhesion = plnToolBox_v2::checkPlugin('adhesion',__CLASS__,'before',false);#v1.2

			# On vérifie que le plugin technique facultatif plnExtendedJurisdiction est installé et chargé avant plnKikan
			self::$isPrivacyPossible = plnToolBox_v2::checkPlugin('plnExtendedJurisdiction',__CLASS__,'before',false);

			# On vérifie que le plugin plxMyMultiLingue est chargé avant plnKikan
			self::$isMultiLingue = plnToolBox_v2::checkPlugin('plxMyMultiLingue',__CLASS__,'before',false);

			# On vérifie que le plugin technique obligatoire plnStaticPages est bien installé et chargé après plnKikan
			self::$isStaticPages = plnToolBox_v2::checkPlugin('plnStaticPages',__CLASS__,'after',true);
		}

		if(defined('PLX_ADMIN'))
		{
			# Pour afficher les éventuelles erreurs verifs des dépandances basiques et avec plnToolBox_v2)
			$this->addHook('AdminTopBottom','AdminTopBottom');
		}

		# Si StaticPages existe, on peut préparer tout le fonctionnement de plnKikan
		if(self::$isStaticPages)
		{
			# Les droits d'accès à la page d'administration du plugin
			switch($this->getParam('droitAcces'))
			{
				case 0  : $this->setAdminProfil(0);break;
				case 1  : $this->setAdminProfil(0,1);break;
				case 2  : $this->setAdminProfil(0,1,2);break;
				case 3  : $this->setAdminProfil(0,1,2,3);break;
				case 4  : $this->setAdminProfil(0,1,2,3,4);break;
				default : $this->setAdminProfil(0);break;
			}
			$this->setAdminMenu($this->getLang('L_ADMIN_MENU'),$this->getParam('adminMeNum'),$this->getLang('L_ADMIN_MENU_NFO'));
			if(defined('PLX_ADMIN'))
			{
				# Pour afficher le css spécifique au datepicker dans la partie admin
				$this->addHook('AdminTopEndHead','AdminTopEndHead');
			}
			else
			{
				#$isPrivate = ($this->getParam('isPrivate') != 'false') ? true : false;// + 404 #v1.2
				#if($this->getParam('isPrivate') != '404'){#v1.2 //AND $this->isGlobalAccessAllowed()
				if($this->getParam('isPrivate') != '404' OR $this->isGlobalAccessAllowed())
				{#v1.2
					# Est-ce qu'un utilisateur est en train de soumettre de nouveaux choix ?
					$this->addHook('plxMotorPreChauffageBegin','manageNewChoice');
					# Pour gérer les demandes d'affichage des kikans côté site.
					$this->addHook('plxMotorPreChauffageBegin','plxMotorPreChauffageBegin');
				}
			}
		}
	}

	##############################################################################################
	#
	# On affiche les messages s'il y a eu des erreurs
	#
	##############################################################################################
	public function AdminTopBottom()
	{
		$where = (basename($_SERVER['PHP_SELF']) == 'parametres_plugins.php');
		if($where){
			$tb = class_exists('plnToolBox_v2') AND function_exists('checkPlugin');#plnToolBoxes (plugin)
			$error = '';
			# Plugin plnStaticPages obligatoire
			if(!self::$isStaticPages)
				$error .= __CLASS__ . ' ' . sprintf($this->getLang('L_DEPENDENCY_ERROR'), '<i>plnStaticPages</i>') . ' ' . $this->getLang('L_DEPENDENCY_END') . PHP_EOL;
			if(!$tb)# ! plnToolBoxes (plugin)
				$error .= __CLASS__ . ' ' . sprintf($this->getLang('L_DEPENDENCY_ERROR'), '<i>plnToolBoxes</i>') . ' ' . $this->getLang('L_DEPENDENCY_TOP');
			else {#plnToolBoxes (plugin)
				# Msg Plugins Optionels
				foreach(explode(' ', 'isMultiLingue lockArticles adhesion isPrivacyPossible') AS $plug)
					if(is_bool(self::$$plug) && !self::$$plug)
						$error .= $this->getLang('L_LOAD_'.strtoupper($plug).'_BEFORE') . ' <i>' . __CLASS__ . '</i>' . PHP_EOL;#plnToolBox_v2::addMsg ou _v1
			}
			if($error AND defined('PLX_ADMIN') AND $_SESSION['profil'] < PROFIL_MANAGER)
				echo '<div class="alert error">'.nl2br($error).'</div>';
		}
	}

	##############################################################################################
	#
	# Pour le pikaday (pikaday.css link)
	#
	##############################################################################################
	public function AdminTopEndHead()
	{
		# On vérifie si on est sur la page admin, ce qui nous permettra d'ajouter les css associés au pikaday
		echo '<?php '; ?>
		if($plxAdmin->path_url && preg_match('~plugin.php\?p=<?=__CLASS__?>&kikanedit~',$plxAdmin->path_url))
		{
			?><link rel="stylesheet" href="<?=PLX_PLUGINS.__CLASS__?>/pikaday/pikaday.css">
<?= '<?php ' ?>
		}
?><?php
	}

	##############################################################################################
	#
	# On récupère le nom du plugin ($this->plug est protected...)
	#
	##############################################################################################
	public function getName()   {return $this->plug['name'];}

	##############################################################################################
	#
	# Est-ce que le visiteur a le droit d'accéder à la liste des kikans ?
	#
	##############################################################################################
	public function isGlobalAccessAllowed()
	{
		# 1 ou 404 = prvate activé
		if($this->getParam('isPrivate') < 1) return true;
		# Si l'utilisateur n'est pas connecté, on le redirige vers l'accueil
		if(
			(self::$lockArticles OR self::$adhesion)
			AND !isset(
				$_SESSION['lockArticles']['articles'],
				$_SESSION['lockArticles']['categorie'],
				$_SESSION['account']
			)
		)
			return false;#v1.2
		if(!self::$isPrivacyPossible)
			return true;
		if(!plnExtendedJurisdiction::isConnected())
			return false;
		if($this->getParam('privateProfiles') != '' and !plnExtendedJurisdiction::isProfile($this->getParam('privateProfiles')))
			return false;
	  return true;
	}

	##############################################################################################
	#
	# Est-ce que le visiteur a le droit d'accéder à un kikan en particulier
	#
	##############################################################################################
	public function isSpecificAccessAllowed($kikanId)
	{
		if(isset($this->aKikan[$kikanId]))
			$Kikan = $this->aKikan[$kikanId];
		else
			return true; // Le kikan n'existe pas, on peut donc bien donner l'accès...

		if( (self::$lockArticles OR self::$adhesion AND !self::$isPrivacyPossible) AND $Kikan['isPrivate'] AND
		   (!isset($_SESSION['lockArticles']['articles']) && !isset($_SESSION['lockArticles']['categorie']) && !isset($_SESSION['account']))
		)
			return false;#v1.2

		if(!self::$isPrivacyPossible or !$Kikan['isPrivate'])
			return true;

		if(!plnExtendedJurisdiction::isConnected())
			return false;
		if($Kikan['privateProfiles'] != '' and !plnExtendedJurisdiction::isProfile($Kikan['privateProfiles']))
			return false;
		return true;
	}

	##############################################################################################
	#
	# Le hook se lançant très tôt et permettant de récupérer les choix de l'utilisateur pour
	# les enregistrer. Est appelé ds plxMotorPreChauffageBegin
	#
	##############################################################################################
	public function manageNewChoice()
	{
		$this->plxMotor = plxMotor::getInstance();#???

		if(isset($_GET[$this->lnGet . __CLASS__]))
		{
			include_once(PLX_ROOT.'core/lib/class.plx.token.php');#kikan.plnKikan.php on line 269
		}
		if(isset($_POST['kikanId']))
		{
			# On ne contrôle le token que si l'utilisateur fait une soumission de choix kikan
			plxToken::validateFormToken($_POST);

			$kikanId = $_POST['kikanId'];

			# On charge les kikan : on sait qu'on va en avoir besoin
			$this->loadKikan(true);#true permet de charger les choix
			$ready = true;
			# Le kikan n'existe pas ou n'est pas publié, on quitte.
			if(!isset($this->aKikan[$kikanId]) OR empty($this->aKikan[$kikanId]['status']))#draft == '0' #old != 'publié'
			$ready = false;

			# Si l'utilisateur n'est pas autorisé à voir ce kikan, on quitte violemment.
			if(!$this->isSpecificAccessAllowed($kikanId))
			$ready = false;

			# Si le nom fourni est vide, on quitte
			if(empty($_POST['name']))
					$ready = false;

			if($ready){
				$aTimestamp = array();
				foreach($this->aKikan[$kikanId]['dates'] as $date => $horaires)
				{
					foreach($horaires as $horaire)
						$aTimestamp[] = ($this->aKikan[$kikanId]['typeKikan'] == 'kikoi' ? plxUtils::title2url($date.'_'.$horaire) : $date.'_'.$horaire);
					if(empty($horaires))
						$aTimestamp[] = ($this->aKikan[$kikanId]['typeKikan'] == 'kikoi' ? plxUtils::title2url($date) : $date);
				}

				# Pour le kikan en cours, on construit la liste des choix de l'utilisateur
				$Choix = array();
				foreach($aTimestamp as $timestamp)
					$Choix[$timestamp] = $_POST[$timestamp];

				# Si un ancien nom est fourni et qu'il est différent, on supprime l'ancien
				if(isset($_POST['old-name']) AND $_POST['name'] != $_POST['old-name'])
					unset($this->aKikan[$kikanId]['choix'][$_POST['old-name']]);#unset old val

				$this->aKikan[$kikanId]['choix'][$_POST['name']] = $Choix;

				# On enregistre juste les Choix
				if($this->saveKikan('choix'))
					plnToolBox_v2::addMsg($this->getLang('L_MSG_SAVE_NEW_CHOICE'),'success');#save msg by session
				else
					plnToolBox_v2::addMsg($this->getLang('L_MSG_SAVE_ERR_CHOICE'),'error');#error msg by session

			}#fi $ready
			unset($_POST);
			#header('Location:'.$this->plxMotor->urlRewrite('?'.$this->plxMotor->get));//adhesion.html
			header('Location:'.$this->plxMotor->racine.$this->plxMotor->path_url);//on redirige
			exit;
		}
	}

	##############################################################################################
	#
	# Méthode callback appelée par plnStaticPages quand on demande un plnKikan
	#
	##############################################################################################
	public function displayKikanFile()
	{
		# Quel fichier veut-on afficher (côté site) ?
		if($this->kikanId === '')
			include('list.'.__CLASS__.'.php');
		else
			include('kikan.'.__CLASS__.'.php');
	}

	public/* static */function AccesInterdit(){
		$this->lang('L_ACCESS_DENIED');
		#$this->plxMotor->error404(L_ERR_PAGE_NOT_FOUND);
	}
	/*
Fatal error: Uncaught Error: Using $this when not in object context in /var/www/0.src.blogs/plx_plugs_git/plnKikan/plnKikan.php on line 409
( ! ) Error: Using $this when not in object context in /var/www/0.src.blogs/plx_plugs_git/plnKikan/plnKikan.php on line 409
Call Stack
#	Time	Memory	Function	Location
1	0.0002	402464	{main}( )	.../index.php:0
2	0.1402	1583960	include( '/var/www/0.src.blogs/PluXml-5.7/themes/defaut-5.7.1-parent/static.php' )	.../index.php:62
3	0.1429	1587752	plxShow->staticContent( )	.../static.php:19
4	0.1560	1604712	require( '/var/www/0.src.blogs/plx_plugs_git/plnStaticPages/dummy.plnStaticPages.php' )	.../class.plx.show.php:1543
5	0.1561	1604712	plnKikan::AccesInterdit( )	.../dummy.plnStaticPages.php:5
*/
	##############################################################################################
	#
	# Lorsque le moteur est en train de préchauffer, on intercepte éventuellement les
	# demandes d'affichage des kikans
	# Cela se fait en cliquant sur "Kikans" et envoie une commande de type index.php?plnKikan
	# ou index.php?plnKikan<num>
	#
	##############################################################################################
	public function plxMotorPreChauffageBegin()
	{
		# Indique à plnStaticPages qu'on gère une "fausse" page statique
		plnStaticPages::newStaticPage(__CLASS__,__CLASS__);

		# Veut-on un lien vers les kikans ? Si oui, dans quelles conditions ?
		if ($this->getParam('inStaticList') == 'true'
		or ($this->getParam('inStaticList') == 'private'
		and $this->isGlobalAccessAllowed()))
			plnStaticPages::setField(__CLASS__,'navLink',str_replace(' ','&nbsp;',$this->getLang('L_MENU_KIS')));


		if(isset($_GET[$this->lnGet . __CLASS__]))
		{
			# Charge les kikan : on sait qu'on va en avoir besoin
			$this->loadKikan(!empty($_GET[$this->lnGet . __CLASS__]));#true permet de charger les choix du kikan

			# Récupère le hash du kikan demandé.
			$hash = plxUtils::strCheck($_GET[$this->lnGet . __CLASS__]);

			$this->kikanId = '';
			foreach($this->aKikan as $kikanId => $Kikan)
				if($Kikan['hash'] == $hash)
				{
					$this->kikanId = $kikanId;
					break;
				}

			if($this->kikanId !== '' and empty($this->aKikan[$kikanId]['status']))#draft == '0' #old != 'publié'
				$this->kikanId  = '';

			# Quel fichier veut-on afficher ?
			$cible = $this->kikanId === '' ? 'list' : 'kikan';

			# Est-ce que l'accès à cette page est autorisé ? Si non, on demande à plnExtendedJurisdiction d'afficher
			# une page indiquant que l'accès est interdit.
			# Si on veut accéder à la liste, il faut vérifier qu'on a l'accès global.
			# Si on veut accéder à un kikan particulier, il faut vérifier qu'on a l'accès à ce kikan en particulier.
			if( ($cible == 'list'  and !$this->isGlobalAccessAllowed())
			 or ($cible == 'kikan' and !$this->isSpecificAccessAllowed($this->kikanId)))
			{
				$this->isAccesInterdit = true;

				if(class_exists('plnExtendedJurisdiction')){
					plnExtendedJurisdiction::AccesInterdit();
				}
				elseif(class_exists('lockArticles') or class_exists('adhesion')){// On déclare une fausse page statique factice #BEP
					plnStaticPages::newStaticPage(__CLASS__, __CLASS__, $this->getLang('L_AUTH_ACCESS'), $this->getLang('L_MENU_KIS') . ' - ' . $this->getLang('L_AUTH_ACCESS_TITLE'), 'AccesInterdit');
					# Et on indique qu'on VEUT l'afficher
					plnStaticPages::$currentPage = __CLASS__;

if(0){/* # code d'exemple
	public static function newStaticPage($class,$trigger,$staticTitle='',$pageTitle='',$method='',$navLink='',$template='')
	{
		self::$aPages[$trigger] = array(
			'trigger'	   => $trigger,	# tel que l'URL appelante est index.php?TRIGGER
			'class'		 => $class,	  # la classe du plugin qui appelle
			'method'		=> $method,	 # la méthode du plugin appelant permettant d'afficher le contenu (optionnel)
			'staticTitle'   => $staticTitle,# le titre de la page statique tel que défini dans le template
			'pageTitle'	 => $pageTitle,  # le nom de la page (apparaitra dans la barre du navigateur)
			'navLink'	   => $navLink,	# le nom du bouton apparaissant dans la navbar. Si vide, pas de bouton
			'template'	  => $template,   # le template à appeler. Si vide, static.php
		);
	}
*/

# Si "method" n'est pas fourni, alors on tentera automatiquement de charger le fichier suivant :
# ../../plugins/<class>/<trigger>.<class>.php
#
# Exemple d'utilisateur à partir d'un plugin nommé PlugEx
#		$this->plxMotor   = plxMotor::getInstance();
#		$plnStaticPages = $this->plxMotor->plxPlugins->aPlugins['plnStaticPages'];
#		$plnStaticPages->newStaticPage('PlugEx','PlugExURL');
#		$plnStaticPages->setField('PlugExURL','staticTitle','Mon titre de page statique');
#		$plnStaticPages->setField('PlugExURL','method','displayPlugExContents');
#		$plnStaticPages->setField('PlugExURL','pageTitle','Mon zouli Titre de barre de navigateur');
#		$plnStaticPages->setField('PlugExURL','navLink','Mon Lien');
#		$plnStaticPages->setField('PlugExURL','template','static.php');
# Dans cet exemple, la fausse page statique sera affichée dès qu'on tentera d'accéder à la page
# index.php?PlugExURL
# Cet affichage se fera via la méthode :
# PlugEx::displayPlugExContents();
# Si jamais on ne renseignait pas la clé "method", alors ce serait le fichier suivant qui serait chargé :
# ../../plugins/PlugEx/PlugExURL.PlugEx.php
}
				}

			}
			else
			{
				$this->isAccesInterdit = false;
				# Quel template ?
				$template = $this->getParam('template')=='' ? 'static.php' : $this->getParam('template');
				# Quel titres afficher ?
				if($this->kikanId === '')
				{
					$pageTitle	  = $this->getLang('L_KIS');
					$staticTitle	= $this->getLang('L_KIS_LIST');
				}
				else
				{
					$staticTitle	= $this->getLang('L_HOME_'.$this->aKikan[$this->kikanId]['typeKikan']);
					$pageTitle	  = $staticTitle.' '.$this->aKikan[$this->kikanId]['title'];
				}
				# On définit l'ensemble des titles et autres composantes accompagnant la page statique
				plnStaticPages::setField(__CLASS__,'staticTitle',$staticTitle);
				plnStaticPages::setField(__CLASS__,'pageTitle',$pageTitle);
				plnStaticPages::setField(__CLASS__,'template',$template);
				plnStaticPages::setField(__CLASS__,'method','displayKikanFile');

				# on renvoit un return true pour qu'il arrête le travail de MotorPrechauffageBegin,
				# puisqu'on sait que ce sera un kikan
				echo '<?php return true ?>';
			}
		}

	}

	##############################################################################################
	#
	# On charge le fichier des kikan, seulement si ce n'est déjà fait.
	#
	##############################################################################################
	public function loadKikan($all = false)
	{
		if(isset($this->aKikan) and is_array($this->aKikan))
			return;

		$this->aKikan = array();

		if(!is_readable($this->DataFile)) return;

		# Mise en place du parseur XML
		$xml = new SimpleXMLElement(implode('',file($this->DataFile)));

		# On parcourt les kikans
		$kikans = $xml->xpath('/document/kikan');
		foreach($kikans as $kikan)
		{
			$temp = array();
			$temp['title']			  = (string)$kikan->title;
			$temp['description']		= (string)$kikan->description;
			$temp['nbDates']			= (string)$kikan->nbDates;
			$temp['nbHoraires']		 = (string)$kikan->nbHoraires;
			$temp['typeKikan']		  = (string)$kikan->typeKikan;#todo type
			$temp['isPrivate']		  = $kikan->isPrivate == 1 ? true : false;
			$temp['privateProfiles']	= (string)$kikan->privateProfiles;
			$temp['status']			 = (string)$kikan->status;
			$temp['dates']			  = array();
			$temp['choix']			  = array();
			$temp['hash']			   = (string)$kikan->hash;

			foreach($kikan->date as $date)
			{
				$temp['dates'][(string)$date['valeur']] = array();
				$j=0;
				foreach($date->horaire as $horaire)
				{
					$temp['dates'][(string)$date['valeur']][$j] = (string)$horaire;
					$j++;
				}
			}

			if($all)
			{
				#$oData = $this->DataFile;#save
				$this->DataFile = str_replace('a.x', 'as_'.$kikan['id'].'.x', $this->DataFile);
				if(is_readable($this->DataFile))
				{
					# Mise en place du parseur XML
					$lmx = new SimpleXMLElement(implode('',file($this->DataFile)));
					# On parcourt les donéées du kikan (choix)
					$didans = $lmx->xpath('/document/kikan');
					foreach($didans as $didan)
					{
						foreach($didan->choix as $choix)
						{
							$temp['choix'][(string)$choix['name']] = array();
							foreach($choix->timestamp as $timestamp)
								$temp['choix'][(string)$choix['name']][(string)$timestamp['valeur']] = (string)$timestamp['choix'];
						}
					}
				}
				#$this->DataFile = $oData;#restaure pour saveKikan() qui peut-être appeler
			}#Fi all
			$this->aKikan[(int)$kikan['id']] = $temp;
		}
	}

	##############################################################################################
	#
	# On sauvegarde le fichier des kikan param $koi 'model ou 'datas'
	#
	##############################################################################################
	public function saveKikan($koi='model')
	{
		# Vérifie le répertoire de langue #Compatibilié plxMyMultiLingue
		if(!is_dir($this->DataDir.$this->lang))
		{
			$oData = $this->DataDir;
			$this->DataDir = $this->DataDir.$this->lang;
			$this->OnActivate();#créer le dossier ##/ + .htaccess
			$this->DataDir = $oData;
		}
		# Début du fichier XML
		$xml = '<?xml version="1.0" encoding="'.PLX_CHARSET.'"?>' . PHP_EOL;
		$xml .= '<document>' . PHP_EOL;
		foreach($this->aKikan as $Id => $kikan)
		{
			$xml .= '	<kikan id="'.$Id.'">' . PHP_EOL;
			switch($koi){
				case 'model':
					# On liste les champs clés + Simple et aucun CDATA (sauf choix et dates)
					foreach(array_keys($kikan) as $k)
						if(!in_array($k, explode(' ','choix dates')))#No dates & choix #Fix admin_kikanedit save :	<choix></choix> & finaly	<choix name=""></choix> && isnogood here
							$xml .= '		<'.$k.'>'.plxUtils::strCheck($kikan[$k]).'</'.$k.'>' . PHP_EOL;
							# title description nbDates nbHoraires typeKikan isPrivate privateProfiles status hash

					ksort($kikan['dates']);
					foreach($kikan['dates'] as $date => $horaires)
					{
						#sort($horaires);
						$xml .= '		<date valeur="'.str_replace(array("'",'"'),array('’','&quot;'),$date).'">' . PHP_EOL;
						foreach($horaires as $horaire)
							$xml .= '			<horaire>'.str_replace("'",'’',$horaire).'</horaire>' . PHP_EOL;
						$xml .= '		</date>' . PHP_EOL;
					}
				break;
				case 'choix':#public edit
					$this->DataFile = str_replace('a.x', 'as_'.$Id.'.x', $this->DataFile);
					# Choix des internautes : Todo séparer les choix en fichiers autonomes like : plnKikan_data_0.xml (prévoir update)
					foreach($kikan['choix'] as $name => $timestamps)
					{
						$xml .= '		<choix name="'.$name.'">' . PHP_EOL;
						foreach($timestamps as $timestamp => $choix)
							$xml .= '			<timestamp valeur="'.$timestamp.'" choix="'.$choix.'" />' . PHP_EOL;
						$xml .= '			</choix>' . PHP_EOL;
					}
				break;
			}#end switch $koi
			$xml .= '	</kikan>' . PHP_EOL;
		}#end foreach aKikan
		$xml .= '</document>' . PHP_EOL;
		return plxUtils::write($xml,$this->DataFile);
	}

	##############################################################################################
	#
	# Créer un nouveau kikan
	#
	##############################################################################################
	public function newKikan($title,$description)
	{
		$this->loadKikan();

		$Kikan=array(
				'title'		 => $title,
				'description'   => $description,
				'private'	   => $private,
				'status'		=> '0',#draft (brouillon)
				'hash'		  => sha1(plxUtils::charAleatoire(15)),
				'dates'		 => array());

		$this->aKikan[] = $Kikan;

		end($this->aKikan);
		$kikanId = key($this->aKikan);
		$this->aKikan[$kikanId]['hash'].=$kikanId;# On rajoute le numéro du kikan pour éviter une très improbable collision
		return($kikanId);
	}

	##############################################################################################
	#
	# Détruit un kikan et les choix des internautes (le fichier datas_#Id#.xml)
	#
	##############################################################################################
	public function deleteKikan($Id)
	{
		$this->loadKikan();
		if(isset($this->aKikan[$Id])){
			unset($this->aKikan[$Id]);
			@unlink(str_replace('a.x', 'as_'.$Id.'.x', $this->DataFile));
		}
	}
}